const { debuggerLog } = require('../utilities');

const validateObject = (obj, scheme, validatorRef = undefined) => {
  let validated = true;
  try {
    Object.keys(obj).forEach((prop) => {
      if (validated) {
        if ((typeof obj[prop]) === 'object' && (scheme[prop] !== undefined && (typeof scheme[prop]) === 'object')) {
          if (scheme[prop] === undefined) {
            validated = false;
            return;
          }

          if (!validateObject(obj[prop], scheme[prop], validatorRef)) {
            validated = false;
          }
        } else if ((typeof obj[prop]) !== scheme[prop]) {
          try {
            if (scheme[prop] === undefined) {
              validated = false;
              return;
            } else if (!validatorRef.check(obj[prop]).is(scheme[prop])) {
              validated = false;
              return;
            }
          } catch (err) {
            throw err;
          }
        }
      }
    });
  } catch (e) {
    throw e;
  }

  return validated;
};

module.exports = require('../interfaces/component.interface')('object', (variable, schema, args, validatorRef) => {
  if (args !== undefined && (typeof args) === 'string') {
    const a = args.split(' ');
    if (a.length !== 1) {
      throw new TypeError(`Scheme name '${args}' must be a single word.`);
    }

    const schemeName = a[0];

    if (schema[schemeName] !== undefined) {
      return validateObject(variable, schema[schemeName], validatorRef);
    }

    throw new TypeError(`Scheme '${schemeName}' not found.`);
  } else if ((typeof args) === 'object') {
    return validateObject(variable, args, validatorRef);
  }

  throw new TypeError(`Invalid type '${typeof args}' passed as argument to performCheck`);
});
